/*
 * main.c
 *
 * A task lets the LED blink. The task can be suspended by button 0 and resumed by button 1.
 *
 * This example is an advanced training material for experienced students.
 *
 * Tasks can be created, killed, suspended and resumed. This example program shows how a
 * how a task is suspended and resumed. We never kill tasks. We use a very simple memory
 * manager and are not able to release the resources used by a task. Note: do not use
 * suspend and resume to synchronize tasks. Use binary semaphores instead.
 *
 * We need two push-buttons for this example. Therefore we will use the reset
 * button as user programmable button. This is possible unless PF6 is configured
 * as a RESET pin. By default it is not configured that way.
 *
 * Please see www.freertos.org for more information.
 * xTaskCreate: FreeRTOS/API Reference/Task Creation
 * vTaskResume: FreeRTOS/API Reference/Task Control
 * vTaskSuspend: FreeRTOS/API Reference/Task Control
 *
 * Created: 2018-07-23
 *  Author: Wolfgang Neff
 */

#include <avr/io.h>
#include <FreeRTOS.h>
#include <task.h>

#include "board.h"

#define LED_TASK_PRIORITY (tskIDLE_PRIORITY+2)
#define LED_DELAY pdMS_TO_TICKS(500)

#define BUTTON_TASK_PRIORITY (tskIDLE_PRIORITY+1)
#define BUTTON_TASK_PERIOD pdMS_TO_TICKS(100)

// We need a task handle to be able to suspend and resume it.
TaskHandle_t xLEDTaskHandle;

void vLEDTask(void* pvParameters)
{
	// Initialize the LED.
	LED_PORT.DIR = LED_PIN_bm;
	LED_PORT.OUT = LED_PIN_bm;
	// Super loop of the task.
	while (1) {
		LED_PORT.OUTTGL = LED_PIN_bm;
		vTaskDelay(LED_DELAY);
	}
}

void vButtonTask(void* pvParameters)
{
	// Initialize the push-buttons.
	BUTTON0_PINCTRL = PORT_PULLUPEN_bm;
    BUTTON1_PINCTRL = PORT_PULLUPEN_bm;
	// Super loop of the task.
	while (1) {
		if (!(BUTTON0_PORT.IN & BUTTON0_PIN_bm)) {
			vTaskResume(xLEDTaskHandle);
		}
		if (!(BUTTON1_PORT.IN & BUTTON1_PIN_bm)) {
			vTaskSuspend(xLEDTaskHandle);
		}
		vTaskDelay(BUTTON_TASK_PERIOD);
	}
}

int main(void)
{
	// Create the tasks.
	xTaskCreate(vLEDTask, "LEDTask", configMINIMAL_STACK_SIZE, NULL, LED_TASK_PRIORITY, &xLEDTaskHandle);
	xTaskCreate(vButtonTask, "ButtonTask", configMINIMAL_STACK_SIZE, NULL, BUTTON_TASK_PRIORITY, NULL);

    // When creating a task it has state ready. We have to suspend it before
	// the scheduler starts if we do not want that it starts automatically. 
    vTaskSuspend(xLEDTaskHandle);

	// Start the scheduler.
	vTaskStartScheduler();
}
