/*
 * main.c
 *
 * This example is an advanced training material for experienced students.
 *
 * Message queues are not limited to simple data types. You can use a struct
 * to pass more complex data. This will be demonstrated here. A button task
 * checks button 0, 1 and lets the LED blink 2 or 6 times.
 *
 * Modified: 2018-07-23
 *   Author: Wolfgang Neff
 */ 

#include <avr/io.h>
#include <FreeRTOS.h>
#include <queue.h>
#include <task.h>

#include "board.h"
#include "switch.h"

#define CAPACITY 8

#define LED_TASK_PRIORITY (tskIDLE_PRIORITY+2)
#define LED_DELAY pdMS_TO_TICKS(250)

#define BUTTON_TASK_PRIORITY (tskIDLE_PRIORITY+1)
#define BUTTON_TASK_DELAY pdMS_TO_TICKS(25)

// Define the data type of the message.
typedef struct {
	uint8_t led;     // The bitmap of the LED to blink.
	uint8_t blinks;  // The number of flashes.
} message_t;

// Declare the message queue.
QueueHandle_t xQueue;

// This task reads the message queue and blinks accordingly.
void vLEDTask(void* pvParameters)
{
	// Declare the message buffer.
    message_t message;
	// Initialize the LEDs.
    LED_PORT.DIR = LED_PIN_bm;
    LED_PORT.OUT = LED_PIN_bm;
	while (1) {
		// Await the message and let the selected led blink.
		xQueueReceive(xQueue, &message, portMAX_DELAY);
		for (uint8_t i=0; i<2*message.blinks; i++) {
			LED_PORT.OUTTGL = message.led;
			vTaskDelay(LED_DELAY);
		}
	}
}

// Check the buttons and send an appropriate message.
void vButtonTask(void* pvParameters)
{
	// Declare the message buffer.
	message_t message;
	// Initialize the push-buttons
	BUTTON0_PINCTRL = PORT_PULLUPEN_bm;
	BUTTON1_PINCTRL = PORT_PULLUPEN_bm;
	while (1) {
		// Check the push-buttons and send the corresponding message.
		if (SWITCH0_PRESSED()) {
			message.led = LED_PIN_bm;
			message.blinks = 2;
			xQueueSend(xQueue, &message, portMAX_DELAY);
		}
		if (SWITCH1_PRESSED()) {
			message.led = LED_PIN_bm;
			message.blinks = 6;
			xQueueSend(xQueue, &message, portMAX_DELAY);
		}
		// Wait a bit before checking the button again.
		vTaskDelay(BUTTON_TASK_DELAY);
	}
}

int main(void)
{
	// Create the message queue.
	xQueue = xQueueCreate(CAPACITY, sizeof(message_t));

	// Create the tasks.
	xTaskCreate(vLEDTask, "LEDTask", configMINIMAL_STACK_SIZE, NULL, LED_TASK_PRIORITY, NULL);
	xTaskCreate(vButtonTask, "ButtonTask", configMINIMAL_STACK_SIZE, NULL, BUTTON_TASK_PRIORITY, NULL);

	// Start the scheduler.
    vTaskStartScheduler();
}
